
# ---------------------------------------------------------------------------
# Copyright (c) 1997 Christian Krone. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Tcl itself.
# See also licence.terms
# ---------------------------------------------------------------------------
# $Id: mkMakefileVC.tcl,v 1.2 1997/01/12 12:58:33 adabas Exp $
# ---------------------------------------------------------------------------

set default {
# ---------------------------------------------------------------------------
# Copyright (c) 1997 Christian Krone. All rights reserved.
#.This program is free software; you can redistribute it and/or
#.modify it under the same terms as Tcl itself.
#.See also licence.terms
# ---------------------------------------------------------------------------
#.Don't edit this file with an editor, better use 'tclsh mkMakefileVC.tcl'...
# ---------------------------------------------------------------------------

# +------------------------------------------------------------------
# Some paths to adjust: the C++ development system, Tcl, Adabas D,...
# -------------------------------------------------------------------
TOOLS    = C:\msdev
TCL      = C:\Program Files\TCL
ADABAS   = D:\adabas95

# +------------------------------------------------------------------
# The current version of Tcl/Tk (as two digit decimal)
# -------------------------------------------------------------------
TCL_VERS = 80
TK_VERS  = 80

# +------------------------------------------------------------------
# Version of Adabas D: 61 for systems <= 6.1, 62 for systems >= 6.2
# -------------------------------------------------------------------
VERSION  = 61

# +------------------------------------------------------------------
# The name of some operating system programs.
# -------------------------------------------------------------------
MAKE     = nmake
RM       = del
MKDIR    = md
INSTALL  = copy
CAT      = type
}

# ---------------------------------------------------------------------------
# readAdabas95mak: read the contents of the status file 'adabas95.mak' and
#                  return a list of all lines. If the file doesn't exist,
#                  return a default list.
# ---------------------------------------------------------------------------
proc readAdabas95mak {} {
    global default

    if [catch {open adabas95.mak} f] {
	set lines $default
    } else {
	set lines [read $f]
	close $f
    }
    return [split $lines "\n"]
}

# ---------------------------------------------------------------------------
# writeAdabas95mak: write the given list into the status file 'adabas95.mak'.
# ---------------------------------------------------------------------------
proc writeAdabas95mak {lines} {
    if [catch {open adabas95.mak w} out] {
	puts stderr $out
	exit 1
    }
    puts $out [join $lines "\n"]
    close $out
}

# ---------------------------------------------------------------------------
# readValues: scan the given list of variable assignments (VAR = value) or
#             layout specs. Put the results in global arrays value and sorted.
# ---------------------------------------------------------------------------
proc readValues {lines}  {
    global value sorted

    set sorted {}
    foreach line $lines {
	if [regexp {^([a-zA-Z0-9_]*) *= *(.*)$} $line match entry val] {
	    set value($entry) $val
	    lappend sorted $entry
	} elseif [regexp {^# \+} $line] {
	    set name sep[llength $sorted]
	    set value($name) +++
	    lappend sorted $name
	} elseif [regexp {^# ([^-].*$)} $line match text] {
	    set value($text) ---
	    lappend sorted $text
	}
    }
}

# ---------------------------------------------------------------------------
# substAdabas95mak: replace all variable assignments in the given list with
#                   the values correspondend to the global array value.
# ---------------------------------------------------------------------------
proc substAdabas95mak {lines}  {
    global value

    foreach line $lines {
	foreach entry [array names value] {
	    if [string compare $value($entry) ---] {
		if [regexp -indices "^$entry *= *" $line indexes] {
		    set    line [string range $line 0 [lindex $indexes 1]]
		    append line $value($entry)
		    break
		}
	    }
	}
	lappend resLines $line
    }
    return $resLines
}

# ---------------------------------------------------------------------------
# writeMakefiles: copy all makefiles of this director hierarchy from
#                 makefile.vc, if they don't exist already.
# ---------------------------------------------------------------------------
proc writeMakefiles {args} {
    foreach dir $args {
	set makefile [file join $dir makefile]
	if ![file exists $makefile] {

            # Note, that it is essantial here to copy the files by reading
            # and writing them, and not by using the Tcl command file copy,
            # since this way the newlines are transformed into crlf by puts
            # automatically.

            if [catch {open $makefile w} out] {
                puts stderr $out
            } elseif [catch {open $makefile.vc} in] {
                puts stderr $in
            } else {
                puts -nonewline $out [read $in]
                close $in
                close $out
	    }
	}
    }
}

# ---------------------------------------------------------------------------
# editValues: Create a toplevel window, which allows the user to change the
#             values of the configuration variables.
# ---------------------------------------------------------------------------
proc editValues {} {
    global value tkCompat okay

    mkMenu    .m
    mkButtons .b
    mkEntries .e
    wm title . "AdabasTcl: Create Makefiles for VC++"
    vwait okay
    wm withdraw .
}

# ---------------------------------------------------------------------------
# mkMenu: Create the menubar of the dialog window.
# ---------------------------------------------------------------------------
proc mkMenu {w} {
    global tk_version

    if {$tk_version >= 8.0} {
	. configure -menu [menu $w]
    	set menu(file) $w.file
	$w add cascade -label "File" -menu $menu(file)  -underline 0

    } else {
	frame $w -relief raised -borderwidth 2
	pack  $w  -side top -fill x

	set menu(file)  $w.file.m
	menubutton $w.file -text "File" -menu $menu(file) -underline 0
        pack $w
    }
    menu $menu(file)
    $menu(file) add command -label "About"  -command about
    $menu(file) add command -label "Okay"   -command {set okay 1}
    $menu(file) add command -label "Cancel" -command exit
}

# ---------------------------------------------------------------------------
# mkEntries: Create a frame with the entries  of the dialog window.
# ---------------------------------------------------------------------------
proc mkEntries {w} {
    global value sorted

    pack [frame $w] -fill x -expand 1
    set row 0
    foreach entry $sorted {
	switch -- $value($entry) {
	    +++ {
		frame $w.lab$row -height 2 -relief sunken -borderwidth 1
		#label $w.lab$row -text ---
		grid $w.lab$row -row $row -column 0 -columnspan 2 -sticky ew \
			-pady 5
	    }
	    --- {
		label $w.lab$row -text $entry
		grid $w.lab$row -row $row -column 0 -columnspan 2 -sticky w
	    }
	    default {
		label $w.lab$entry -text $entry
		entry $w.ent$entry -textvariable value($entry)
		grid  $w.lab$entry -row $row -column 0 -sticky w
		grid  $w.ent$entry -row $row -column 1 -sticky ew -padx 10 -pady 1
		grid  columnconfigure $w 1 -weight 100
	    }
	}
	incr row
    }

    return $w
}

# ---------------------------------------------------------------------------
# mkButtons: Create the buttons at the bottom of the dialog window.
# ---------------------------------------------------------------------------
proc mkButtons {w} {
    pack [frame $w] -side bottom
    button $w.ok  -text "Okay"   -command {set okay 1}
    button $w.can -text "Cancel" -command exit
    pack $w.ok $w.can -side left -padx 5 -pady 5 -fill both
    return $w
}

# ---------------------------------------------------------------------------
# about: Display a few words about Krischan.
# ---------------------------------------------------------------------------
proc about {} {
    tk_messageBox -title "About AdabasTcl" -message "AdabasTcl
Version 0.6\n\nJanuary 1997\n\nChristian Krone
krischan@cs.tu-berlin.de" -type ok
}

# ---------------------------------------------------------------------------
# The main program
# ---------------------------------------------------------------------------
catch {console hide}
set lines [readAdabas95mak]
readValues $lines
editValues
set lines [substAdabas95mak $lines]
writeAdabas95mak $lines
writeMakefiles . extension applications manuals
exit
